'' =================================================================================================
''
''   File....... jm_ds1822_pst.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2009-2022 Jon McPhalen
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 27 JAN 2022
''
''   {$P1}
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate

  #0, T_PST, T_ANSI                                             ' terminal types

  T_TYPE = T_PST


con { fixed io pins }

  RX_PGM   = 31  { I }                                          ' serial / programming
  TX_PGM   = 30  { O }

  SDA_EE   = 29  { I/O }                                        ' i2c / eeprom
  SCL_EE   = 28  { I/O }


con { app io }

  OWX      =  0  { I/O }


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  ow   : "jm_1-wire"                                            ' * 1-Wire driver
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
  ansi : "jm_ansi"                                              '   ANSI terminal control sequences

' * uses cog when loaded


pub main | status, t0, tmpC, tmpF

  setup

  wait_for_terminal(true)                                       ' open pst, enable, press a key
  time.pause(100)

  status := ow.reset                                            ' check for device

  if (status == %10)                                            ' good 1W reset
    read_sn(@SN0)                                               ' read the serial #
    term.tx(term#CLS)
    term.fstr1(string("(%.2x) "), SN0[7])
    repeat t0 from 6 to 1
      term.fxhex(SN0[t0], 2)
      term.tx(" ")
    term.fstr1(string("[%.2x]"), SN0[0])

    term.txn(term#CR, 2)
    term.str(string("Temp", 13, "-------", 13))

    t0 := cnt
    repeat
      tmpC := read_tc                                           ' read the temperature
'     tmpC := read_tca(@SN0)                                    ' read from specific address
      term.fstr0(string("\002\000\004"))
      term.fstr1(string("%6.1fC"), tmpC/1000)                   ' display in C
      term.tx(term#CR)
      tmpF := tmpC * 9 / 5 + 32_0000                            ' convert to Fahrenheit
      term.fstr1(string("%6.1fF"), tmpF/1000)                   ' display in F
      waitcnt(t0 += CLK_FREQ)

  else
    term.tx(term#CLS)
    case status
      %00 : term.str(string("Buss Short"))
      %01 : term.str(string("Buss Error"))
      %11 : term.str(string("No Device"))

    repeat
      waitcnt(0)                                                ' wait for reset


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  ow.start(OWX)                                                 ' start 1-Wire

  term.tstart(BR_TERM)                                          ' start serial for terminal


pub wait_for_terminal(clear)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    clear_screen


pub clear_screen

  if (T_TYPE == T_PST)
    term.tx(term#CLS)
  else
    term.str(ansi.hide_cursor)
    term.str(ansi.home)
    term.str(ansi.cls)


con

  { ------------- }
  {  1 - W I R E  }
  { ------------- }


con

  RD_ROM     = $33                                              ' 1W ROM commands
  MATCH_ROM  = $55
  SKIP_ROM   = $CC
  SRCH_ROM   = $F0
  ALARM_SRCH = $EC

  CVRT_TEMP  = $44                                              ' DS1822 commands
  WR_SPAD    = $4E
  RD_SPAD    = $BE
  COPY_SPAD  = $48
  RD_EE      = $B8
  RD_POWER   = $B4


dat

  SN0           byte    $00, $00, $00, $00, $00, $00, $00, $00


pub read_sn(p_sn)

'' Reads serial number from 1W device
'' -- stores in array at p_sn

  ow.reset
  ow.write(RD_ROM)
  repeat 8
    byte[p_sn++] := ow.read


pub read_tc | tmpC

'' Reads temperature from DS1822
'' -- returns degrees C in 0.0001 degree units

  ow.reset
  ow.write(SKIP_ROM)                                            ' no serial number needed
  ow.write(CVRT_TEMP)                                           ' start temp conversion
  repeat                                                        ' let conversion finish
    tmpC := ow.rdbit
  until (tmpC == 1)
  ow.reset
  ow.write(SKIP_ROM)
  ow.write(RD_SPAD)                                             ' read scratchpad
  tmpC := ow.read                                               ' lsb of temp
  tmpC |= ow.read << 8                                          ' msb of temp

  tmpC := ~~tmpC * 625                                          ' extend sign, 0.0001 units

  return tmpC


pub read_tca(p_sn) | tmpC, idx

'' Reads temperature from DS1822 at specific address
'' -- returns degrees C in 0.0001 degree units
'' -- p_sn is pointer to byte array with serial #

  ow.reset
  ow.write(MATCH_ROM)                                           ' use serial #
  repeat idx from 0 to 7                                        ' send serial #
    ow.write(byte[p_sn][idx])
  ow.write(CVRT_TEMP)                                           ' start conversion
  repeat                                                        ' let conversion finish
    tmpC := ow.rdbit
  until (tmpC == 1)
  ow.reset
  ow.write(MATCH_ROM)
  repeat idx from 0 to 7
    ow.write(byte[p_sn][idx])
  ow.write(RD_SPAD)                                             ' read scratchpad
  tmpC := ow.read                                               ' lsb of temp
  tmpC |= ow.read << 8                                          ' msb of temp

  tmpC := ~~tmpC * 625                                          ' extend sign, 0.0001 units

  return tmpC


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}